structure Stats =
struct
  type u = {usr:Time.time,sys:Time.time,tot:Time.time}
  type t = {nongc:u,gc:u,total:u}
	   
  val zero =
      let
        val zeroU = {usr = Time.zeroTime, 
                     sys = Time.zeroTime, 
                     tot = Time.zeroTime}
      in
        {nongc = zeroU, gc = zeroU, total = zeroU}
      end

  fun wrap (f : 'a -> 'b, arg: 'a) : ('b * t) =
    let
      fun toMS s = s
      val cpu_timer = Timer.startCPUTimer ()
      val res = f arg
      val {nongc,gc} = Timer.checkCPUTimes cpu_timer
      val nongc_usr = toMS (#usr nongc)
      val nongc_sys = toMS (#sys nongc)
      val nongc_tot = Time.+ (nongc_usr, nongc_sys)
      val gc_usr = toMS (#usr gc)
      val gc_sys = toMS (#sys gc)
      val gc_tot = Time.+ (gc_usr, gc_sys)
      val total_usr = Time.+ (nongc_usr, gc_usr)
      val total_sys = Time.+ (nongc_sys, gc_sys)
      val total_tot = Time.+ (total_usr, total_sys)
    in
      (res,
       {nongc = {usr = nongc_usr, sys = nongc_sys, tot = nongc_tot},
        gc = {usr = gc_usr, sys = gc_sys, tot = gc_tot},
        total = {usr = total_usr, sys = total_sys, tot = total_tot}})
    end

  fun toString ({nongc, gc, total} : t) =
    concat [

            "nongc: ",
            "usr: ", (Time.toString (#usr nongc)), "  ",
            "sys: ", (Time.toString (#sys nongc)), "  ",
            "total: ", (Time.toString (#tot nongc)), "\n",
            "gc: ", 
            "usr: ", (Time.toString (#usr gc)), "  ",
            "sys: ", (Time.toString (#sys gc)), "  ",
            "total: ", (Time.toString (#tot gc)), "\n",

            "total: ", 
            "usr: ", (Time.toString (#usr total)), "  ",
            "sys: ", (Time.toString (#sys total)), "  ",
            "total: ", (Time.toString (#tot total))]



end
