(**********************************************************************
 ** adaptive_cps.sml
 **
 ** Copyright 2007 Umut A. Acar, Matthew Fluet, Ruy Ley-Wild
 **
 ** This program is distributed under the terms of the GNU General
 ** Public License as published by the Free Software Foundation.  See
 ** the LICENSE file for the terms and conditions.
 **
 ***************
 ** Description: The implementation of the adaptive library
 **********************************************************************)

structure AdaptiveCPS :> ADAPTIVE_CPS =
struct
   structure SAC = SelfAdjustCore

   (* Meta *)
   structure Box =
      struct
         type 'a t = 'a SAC.Modref.t
         val eq = SAC.Modref.eq
         val hash = SAC.Modref.hash
         val key = SAC.Modref.key
         fun toString vToString b = 
            (vToString (SAC.deref b)) ^ "@" ^ (SAC.Modref.toString b)
      end
   type 'a box = 'a Box.t

   val newEq : 'a Eq.t * 'a -> 'a box = SAC.new
   val new : 'a -> 'a box = fn v => SAC.new (Eq.default,v)
   val change = SAC.change
   val deref = SAC.deref

   val init = SAC.init
   val propagate = SAC.propagate
   val stats = SAC.stats

   (* CPS *)
   type 'a cont = 'a SAC.cont

   structure Dest =
      struct
         type 'a t = 'a SAC.Dest.t
      end

   val alloc = SAC.alloc
   val write = SAC.write
   val read = SAC.read
   val hasNoReads = SAC.hasNoReads

   structure AllocWriteMemoTable =
      struct
         structure T = SAC.AllocWriteTable
         structure Dest = T.Dest

         structure Table = T.Table

         val alloc = T.alloc
         val write = T.write

         val allocWrite = T.allocWrite
      end

   structure FunMemoTable =
      struct
         structure T = SAC.MemoTable
         val mkFunOne = T.mkMemoFunOne
         val mkFunTwo = T.mkMemoFunTwo
         val mkFun = T.mkMemoFun
      end
   structure ContMemoTable =
      struct
         structure T = SAC.MemoTable
         val mkCont = T.mkMemoCont
      end

   (* CPS-Meta *)
   exception InitialRun
   type ('a, 'b) call_fn = ('a * 'b cont) cont
   val call = fn (f : ('a, 'b) call_fn, arg : 'a) =>
      let
         val r = ref (Result.Exn InitialRun)
         fun finish res = r := res
         val k0 = fn (v : 'b) => finish (Result.Value v)
         val () = f (arg, k0)
      in
         r
      end
end

structure AdaptiveMeta : ADAPTIVE_META = AdaptiveCPS
