(**********************************************************************
 ** hash.sml
 **
 ** Copyright 2008 Umut A. Acar, Matthew Fluet, Ruy Ley-Wild
 **
 ** This program is distributed under the terms of the GNU General
 ** Public License as published by the Free Software Foundation.  See
 ** the LICENSE file for the terms and conditions.
 **
 ***************
 ** Description: The implementaion of hash functions
 **********************************************************************)

structure Hash :> HASH =
struct
   type 'a t = 'a -> word
   val default = MLton.hash

   val bool: bool t = default
   val exn: exn t = default
   val int: int t = default
   val list = fn hashA => fn al =>
      let
         val al = (List.take (al, 5)) handle Subscript => al
      in
         default (List.map hashA al)
      end
   val option = fn hashA => fn ao =>
      default (Option.map hashA ao)
   val real: real t = default
   val string: string t = default
   val sum = fn (hashA,hashB) => fn abs =>
      default (Sum.map (hashA,hashB) abs)
   val tuple2 = fn (hashA,hashB) => fn (a,b) =>
      default (hashA a, hashB b)
   val tuple3 = fn (hashA,hashB,hashC) => fn (a,b,c) =>
      default (hashA a, hashB b, hashC c)
   val tuple4 = fn (hashA,hashB,hashC,hashD) => fn (a,b,c,d) =>
      default (hashA a, hashB b, hashC c, hashD d)
   val unit = fn () => default ()
   val word: word t = default
end
