(**********************************************************************
 ** key.sml
 **
 ** Copyright 2008 Umut A. Acar, Matthew Fluet, Ruy Ley-Wild
 **
 ** This program is distributed under the terms of the GNU General
 ** Public License as published by the Free Software Foundation.  See
 ** the LICENSE file for the terms and conditions.
 **
 ***************
 ** Description: The implementation of hash-table key functions
 **********************************************************************)

structure Key :> KEY =
struct
   type 'a t = {eq: 'a Eq.t, hash: 'a Hash.t}
   val default = {eq = Eq.default, hash = Hash.default}

   val bool = {eq = Eq.bool, hash = Hash.bool}
   val exn = {eq = Eq.exn, hash = Hash.exn}
   val int = {eq = Eq.int, hash = Hash.int}
   val list = fn {eq, hash} =>
      {eq = Eq.list eq,
       hash = Hash.list hash}
   val option = fn {eq, hash} =>
      {eq = Eq.option eq,
       hash = Hash.option hash}
   val real = {eq = Eq.real, hash = Hash.real}
   val string = {eq = Eq.string, hash = Hash.string}
   val sum = fn ({eq = eqA, hash = hashA},
                 {eq = eqB, hash = hashB}) => 
      {eq = Eq.sum (eqA, eqB),
       hash = Hash.sum (hashA, hashB)}
   val tuple2 = fn ({eq = eqA, hash = hashA},
                    {eq = eqB, hash = hashB}) => 
      {eq = Eq.tuple2 (eqA, eqB), 
       hash = Hash.tuple2 (hashA, hashB)}
   val tuple3 = fn ({eq = eqA, hash = hashA},
                    {eq = eqB, hash = hashB},
                    {eq = eqC, hash = hashC}) => 
      {eq = Eq.tuple3 (eqA, eqB, eqC), 
       hash = Hash.tuple3 (hashA, hashB, hashC)}
   val tuple4 = fn ({eq = eqA, hash = hashA},
                    {eq = eqB, hash = hashB},
                    {eq = eqC, hash = hashC},
                    {eq = eqD, hash = hashD}) => 
      {eq = Eq.tuple4 (eqA, eqB, eqC, eqD), 
       hash = Hash.tuple4 (hashA, hashB, hashC, hashD)}
   val unit = {eq = Eq.unit, hash = Hash.unit}
   val word = {eq = Eq.word, hash = Hash.word}
end
