(**********************************************************************
 ** self-adjust-core.sig
 **
 ** Copyright 2008 Umut A. Acar, Matthew Fluet, Ruy Ley-Wild
 **
 ** This program is distributed under the terms of the GNU General
 ** Public License as published by the Free Software Foundation.  See
 ** the LICENSE file for the terms and conditions.
 **
 ***************
 ** Description: The signature for the core self-adjusting code
 **********************************************************************)

signature SELF_ADJUST_CORE =
sig
   type 'a cont = 'a -> unit
 
   (**********************************************************************
    ** Modifiables
    **********************************************************************)
   structure Modref :
      sig
         type 'a t
         val eq: 'a t Eq.t
         val hash: 'a t Hash.t
         val key: 'a t Key.t
         val toString: 'a t -> string
      end
   type 'a modref = 'a Modref.t
   structure Dest :
      sig
         type 'a t
      end
   type 'a dest = 'a Dest.t
   val alloc: ('a Eq.t * 'a dest cont) cont
   val write: ('a dest * 'a * 'a modref cont) cont
   val read: ('a modref * 'a cont) cont


   (**********************************************************************
    ** Memoization
    **********************************************************************)
   structure MemoTable :
      sig
         val mkMemoFunOne: 'arg Key.t ->
                           ('arg * ('res cont * exn cont) modref) cont ->
                           ('arg * 'res cont * exn cont) cont
         val mkMemoFunTwo: 'arg Key.t ->
                           ('arg * ('res cont modref * exn cont modref)) cont ->
                           ('arg * 'res cont * exn cont) cont
         val mkMemoFun: 'arg Key.t -> 'arg cont -> 'arg cont
         val mkMemoCont: 'arg Eq.t -> 'arg cont -> 'arg cont
      end

   structure AllocWriteTable :
      sig
         structure Dest :
            sig
               type 'a t
            end

         structure Table :
            sig
               type ('key,'a) t
               val make : 'key Key.t -> ('key, 'a) t
            end

         val alloc : ('a Eq.t * 'a Dest.t cont) cont
         val write : ('key,'a) Table.t * 'key -> ('a Dest.t * 'a * 'a modref cont) cont

         val allocWrite: ('key,'a) Table.t * 'key -> ('a Eq.t * 'a * 'a modref cont) cont
      end

   (**********************************************************************
    ** Meta operations
    **********************************************************************)
   val init: unit -> unit
   val new: 'a Eq.t * 'a -> 'a modref
   val change: 'a modref * 'a  -> unit
   val deref: 'a modref -> 'a
   val propagate: unit -> int * bool
   val hasNoReads: 'a modref -> bool
   val stats: unit -> string list

end
