(**********************************************************************
 ** GEOMETRY_TWO_D.sml
 **
 ** Copyright 2005 Umut A. Acar
 **
 ** Written by Umut Acar, Kanat Tangwongsan, Jorge Vittes
 **
 ** This program is distributed under the terms of the GNU General
 ** Public License as published by the Free Software Foundation.  See
 ** the LICENSE file for the terms and conditions.
 **
 ***************
 ** Description: Primitives for two-dimensional geometry.
 **********************************************************************)

signature GEOMETRY_TWO_D =
 sig

   structure Point : POINT_TWO_D
   structure Line : sig type t = Point.t * Point.t end

   val newLine: Point.t*Point.t -> Line.t
   val lineSideTest: Line.t*Point.t -> Polynomial.t			         

   (** Be careful with toLeft because it makes an arbitrary
       decisions when two points have the same x coordinate 
    **)
   val toLeft: Point.t*Point.t -> Polynomial.t
   val toLeftOrBelow: Point.t*Point.t -> Polynomial.t
   val toLeftOrAbove: Point.t*Point.t -> Polynomial.t

   (** Be careful with because it makes an arbitrary
       decisions when two points have the same y coordinate 
    **)
   val toRight: Point.t*Point.t -> Polynomial.t
   val toRightOrBelow: Point.t*Point.t -> Polynomial.t
   val toRightOrAbove: Point.t*Point.t -> Polynomial.t

   val minY: Point.t*Point.t -> Polynomial.t
   val maxY: Point.t*Point.t -> Polynomial.t

   val minX: Point.t*Point.t -> Polynomial.t
   val maxX: Point.t*Point.t -> Polynomial.t

   val distance: Point.t * Point.t -> Polynomial.t
   val distanceToLine: Point.t*Line.t -> Polynomial.t
   val distCmp: Point.t*Point.t*Line.t -> Polynomial.t

   val distPair: ((Point.t*Point.t)*(Point.t*Point.t)) -> Polynomial.t
 end
