(**********************************************************************
 ** Test.sml
 **
 ** Copyright 2005 Umut A. Acar
 **
 ** Written by Umut A. Acar Kanat Tangwongsan, Jorge Vittes
 **
 ** This program is distributed under the terms of the GNU General
 ** Public License as published by the Free Software Foundation.  See
 ** the LICENSE file for the terms and conditions.
 **
 ***************
 ** Description: Primives for two-dimensional geometry.
 **********************************************************************)
structure GeometryTwoD :> GEOMETRY_TWO_D =
 struct
   structure Poly = Polynomial

   structure Point = PointTwoD
   structure Line = struct type t = Point.t*Point.t end

   fun newLine (p1,p2) = (p1,p2)

   (** Isn't this buggy? **)
   val lineSideTest =
    fn ((p1,p2), p) =>
       Point.crossL ({p1=p2,p2=p},p1)

(** Should line side test be implemented like this?
   val lineSideTest =
    fn ((p1,p2), p) => 
       let
	 val uvec = Point.xysub (p2,p1)
	 val vvec = Point.xysub (p,p1)
       in
	 Point.cross (uvec,vvec)
       end
**)


   val toLeftOrAbove =
    fn (p,q) =>
       let val xtest = Point.xsub (q,p)
       in if Poly.isZero xtest
          then (*XXX why not ysub(p,q)?*)
               Point.ysub (q,p)
          else xtest
       end
   val toLeftOrBelow =
    fn (p,q) =>
       let val xtest = Point.xsub (q,p)
       in if Poly.isZero xtest
          then (*XXX why not ysub(p,q)?*)
               Point.ysub (p,q)
          else xtest
       end
   val toLeft = toLeftOrAbove

   val toRightOrAbove =
    fn (p,q) =>
       let val xtest = Point.xsub (p,q)
       in if Poly.isZero xtest
          then Point.ysub (p,q)
          else xtest
       end
   val toRightOrBelow =
    fn (p,q) =>
       let val xtest = Point.xsub (p,q)
       in if Poly.isZero xtest
          then Point.ysub (q,p)
          else xtest
       end
   val toRight = toRightOrAbove

  (* minY: (point*point) -> bool * real list
   *
   * returns true if p1 is lower (lower y value) than p2 at the current
   * time, it calls Adap.compare to calculate it
   *)	
  fun minY (p1:Point.t,p2:Point.t) = 
      Point.ysub (p2,p1)

  (* maxY: (point *point) -> bool * real list
   *
   * returns true if p1 is higher (higher y value) than p2 at the current
   * time, it calls Adap.compare to calculate it
   *)	
  fun maxY (p1:Point.t,p2) = 
      Point.ysub (p1,p2)

  (* minX: (point * point) -> bool * real list
   *
   * if a is left of b it returns true, else returns false
   *)
  fun minX (p1:Point.t,p2) =
      Point.xsub (p2,p1)

  (* maxX: (point * point) -> bool * real list
   *
   * if a is right of b it returns true, else returns false
   *)
  fun maxX (p1:Point.t,p2) = 
      Point.xsub (p1,p2)

  fun distanceToLine (p, (p1,p2)) =
      Point.crossR (p,{p1=p1,p2=p2})

  (* distCmp: (point * point * (point*point)) -> bool * real list
   ap,bp are points and line is just a line

   This returns whether or not ap is further from the line than
   bp, and adds failure times to the queue for invalidating these
   "certificates" if neccessary
   *)
  fun distCmp (ap:Point.t,bp,line) =
      Poly.sub(distanceToLine(ap, line), distanceToLine(bp,line))


  fun distance (a,b) =
      let 
	  val (xdiff,ydiff) = Point.xysub (a,b)
      in
	  Poly.add(Poly.mult(xdiff,xdiff),Poly.mult(ydiff,ydiff))
      end

  fun distPair ((a,b),(c,d)) =
      let
	  val d1 = distance(a,b)
	  val d2 = distance(c,d)
      in
	  Poly.sub(d1,d2)
      end

 end
