(**********************************************************************
 ** LeftistHeap.sml
 **
 ** Umut Acar
 **
 ** This code taken from Purely Functional Data Structure (p. 20), 
 ** by Chris Okasaki.
 **********************************************************************)

functor LeftistHeap (structure Priority : TOTAL_ORD) =
struct
type elt = Priority.t

datatype node = E | T of int * elt * node * node
type pqueue = int * node

exception Empty

val DEBUG = false
fun dPrint str = 
  if DEBUG then 
    print str
  else 
    ()


fun size (n,_) = n

fun rank E = 0
  | rank (T(r, _, _, _)) = r

fun makeheap (x, a, b) =
  if rank a >= rank b then
    T (rank b + 1, x, a, b)
  else
    T (rank a + 1, x, b, a)

val empty = (0,E)

fun isEmpty (0,E) = true
  | isEmpty (_,T _) = false

fun merge ((n1,q1), (n2,q2)) = 
  let
    fun m (h, E) = h
      | m (E, h) = h
      | m (h1 as T (_, x, a1, b1), h2 as T (_, y, a2, b2)) =
	(case Priority.compare(x,y) of
	   LESS => makeheap (x, a1, m (b1, h2))
	 | _    => makeheap (y, a2, m (h1, b2)))
  in
    (n1+n2,m (q1,q2))
  end



fun insert (x,(n,h)) = 
  let
    fun ins (x, E) = T (1, x, E, E)
      | ins (x, h as T (r, y, a, b)) =
	(case Priority.compare(x,y) of
	   LESS => T (r + 1, x, h, E)
	 | _    => makeheap (y, a, ins (x, b)))
  in
    (n+1,ins (x,h))
    
  end

fun findMin (_,E) = NONE
  | findMin (_,T (_, x, _, _)) = SOME(x)

fun deleteMin (_,E) = raise Empty
  | deleteMin (n, T (_, x, a, b)) = 
    let
      fun m (h, E) = h
	| m (E, h) = h
	| m (h1 as T (_, x, a1, b1), h2 as T (_, y, a2, b2)) =
	  (case Priority.compare(x,y) of
	     LESS => makeheap (x, a1, m (b1, h2))
	   | _    => makeheap (y, a2, m (h1, b2)))
    in
      ((x, (n-1, m (a, b))))
    end

fun discardMin h = 
  let val (_, h') = deleteMin h
  in h'
  end	      
end

