functor BuildListInOut(structure Elt : ELT
                      structure NAElt : ELT
		      val convertElt: Elt.t -> NAElt.t) =
struct

  fun eltListToString l = 
    let val str = List.foldl 
		    (fn (h,str) =>  (Elt.toString h) ^ ", " ^ str) 
		    "nil"
		    l
    in
      "[" ^  str ^ "]"
    end

  fun naeltListToString l = 
    let val str = List.foldl 
		    (fn (h,str) =>  (NAElt.toString h) ^ ", " ^ str) 
		    "nil"
		    l
    in
      "[" ^  str ^ "]"
    end

   structure Adaptive = 
      struct 
         type t = Elt.t ListAdaptive.t
         val toString = ListMetaAdaptive.toString Elt.toString
      end
   structure NonAdaptive = 
      struct
         type t = NAElt.t ListNonAdaptive.t
         val toString = ListMetaNonAdaptive.toString NAElt.toString
      end
   structure Converter =
      struct
         type a_t = Adaptive.t
         type na_t = NonAdaptive.t
         fun convert (l : a_t) : na_t =
            case ListMetaAdaptive.deref l of
               ListAdaptive.NIL => ListMetaNonAdaptive.newEq (NAElt.eq, ListNonAdaptive.NIL)
             | ListAdaptive.CONS (x,t) =>
                  ListMetaNonAdaptive.newEq (NAElt.eq, ListNonAdaptive.CONS (convertElt x, convert t))
      end
   structure Checker =
      struct
         type a_t = Adaptive.t
         type na_t = NonAdaptive.t
         fun check (l : a_t, l' : na_t) =
            case (ListMetaAdaptive.deref l, ListMetaNonAdaptive.deref l') of
               (ListAdaptive.NIL, ListNonAdaptive.NIL) => true
             | (ListAdaptive.CONS (x,t), ListNonAdaptive.CONS (x',t')) =>
	       let
(*
		 val _ = print ("Comparing: " ^ NAElt.toString (convertElt x) ^ 
				" with " ^ NAElt.toString x')
                  val r = NAElt.eq (convertElt x, x')
		  val _ = print (" outcome = " ^ (Bool.toString r) ^ "\n")
*)
	       in
		 NAElt.eq (convertElt x, x') andalso check (t, t')
	       end
						 
             | _ => (false)
      end
end
